/* asaai.h
 * Copyright (C) 2004 Djukic Marko, Pernus Klemen, Zagar Miha
 * For conditions of distribution and use, see the copyright notice in main.cpp
 */
#ifndef _ASAAI_H_
#define _ASAAI_H_

#include "point.h"
#include "trace.h" //kjer se nahaja sled

#include "llist.h" //link lista

#include "objects.h" //okjetki (crta)
#include "constraints.h" //omejitve (stik, vzporednost, kot, dolzina)

#include "ai.h" //skrbi za ucenje in parametre
#include "recognizer.h" //skrbi za prepoznavo objekotv
#include "constraint_manager.h" //skrbi za pavilno umestitev obektov


#include "razno.h" //razred za stanje miske, ...

class Asaai {
private:
	//canvas, kjer se vse dogaja
	FXCanvas *canvas;
	//text button ... za vnos texka
	FXButton *text_button;
	//stvar ki se zna uciti in povedat kaj je OK in kaj ni
	AI *ai;
	//in to je stvar, ki zna prepoznati kaj je bilo narisano
	Recognizer *recognizer;
	//ta zadeva pa skrbi za omejitve
	Constraint_Manager *constraint_manager;


	//tisto kar je uporabnik nazadnje narisal
	Trace *last_input; 

	//oznaci vse objekte v listi objektov kot neoznacene
	void unselect_objects();
	//oznaci vse omejitve v listi omejitev kot neoznacene
	void unselect_constraints();

	//narise spremembo na ekran in v last input
	void update_screen();
public:
	//lista objektov, ki so trenutno narisani na ekranu
	list <Object*> object_list;
	//lista omejitev, ki trenutno delujejo na objekte narisane na ekranu
	list <Constraint*> constraint_list;

	//miska vsebuje vse vitalne informacije o trenutnem stanju ... ja, miske!
	Mouse *mouse_state;
	//ali so constrainti vidni ali ne
	unsigned char show_constraints;
	//vrne ai modul
	AI *get_ai();

	//vrne recogniserja
	Recognizer *get_recognizer();

	//konstruktorcek ...
	Asaai(FXCanvas *_canvas, FXButton *_text);

	//pobrise vse objekte in omejitve ... prazne ekran
	void clear();

	//ko uporabnik pritisne levi gumb
	//I. odoznaci vse objekte
	//II. rezervira pristor za novo sled
	//III. si oznaci koordiante pritiska, in pritisk
	//IV. narise spremembo
	void on_lbutton_down(Point *pozicija);
	//ko uporabnik spusti levi gumb
	//I. popravi koordinate miske
	//II. narise spremembo
	//III. poslje v obdelavo in obdela zadevo
	void on_lbutton_up(Point *pozicija);
	//ko uporabnik pritisn desni gumb
	//I. odoznacim vse objekte
	//se najprej sprehodim cez objekte, ce je kaksen kliknen
	//in ce ni se cez omejitve ce je kaksna izbrana
	void on_rbutton_down(Point *pozicija);
	//ko uporabnik spusti desni gumb
	//ce je drzal omejitev, pogledam ce jo se vedno drzi
	//in ce jo jo izbrisem
	void on_rbutton_up(Point *pozicija);

	//pogleda ce je bil levi dvoni klik na 
	//oznacenem objektu in ce je bil, potem
	//na tem mestu lomi objekt
	void on_lbutton_double(Point *pozicija);
	//pogleda ce je bil desni dvojni klik na
	//objektu in ce je bil ga izbrise
	void on_rbutton_double(Point *pozicija);


	//se klice ob premiku miske 
	//ce gre za risanje po ekranu -> rise po ekranu
	//drugace pogleda ce gre za premikanje in premakne objekt
	void on_mouse_move(Point *pozicija);

	//se klice ob paint eventu
	void on_paint();
};



#endif //_ASAAI_H_