/* constraints.h
 * Copyright (C) 2004 Djukic Marko, Pernus Klemen, Zagar Miha
 * For conditions of distribution and use, see the copyright notice in main.cpp
 */
#ifndef _CONSTRAINT_H_
#define _CONSTRAINT_H_

#include "primitive.h"
#include "objects.h"
#include "point.h"

class Line;

// *****************************************
//
//	Razred CONSTRAINT ... oce vsem omejitvam
//
// *****************************************
class Constraint: public Primitive {
protected:
	Object *object1, *object2;
	Line *line1, *line2;
	//tip omejitve (1 - stik, 2 - vzporednost)
	int type;
	//ali je omejitev uveljavljena, tj. je res, da sta crti npr. vzporedni
	int uveljavljen;
	//ne more biti uveljavljen na kakrsn koli nacin
	int broken;
	//flag, ki ga uporabljam pri premikanju
	int visited_on_move;

public:
	//konstruktorcek
	Constraint();
	~Constraint();
	//vrne objekt z stevilko obj ki ga drzi omejitev
	Object *get_object(int obj);
	//tip omejitve
	int get_constraint_type();
	//vrne drugi objekt glede na podanega
	Object *get_other_object(Object *o);
	//vrne crto namesto objekta
	class Line *get_other_line(class Line *l);
	//dobimo uveljavljen flag
	int get_u(){return uveljavljen;};
	//postavljamo uveljavljen flag
	void set_u(int i){uveljavljen = i;};
	//broken flag
	int get_broken() {return broken;};
	//postavi broken flag
	void set_broken(int i) {broken = i;};
	//vrne flag
	int get_visited_on_move() {return visited_on_move;};
	//postavi flag
	void set_visited_on_move(int v){visited_on_move = v;};
	//doda se k objektu/om, ki ga/jih omejuje
	virtual void add_to_objects() = 0;
	//izbris omejitve iz objektov
	virtual void remove_from_objects() = 0;
};


// *************************************
//
//	Razred CONNECTION
//
// *************************************
class Connection: public Constraint {
private:
	//tocka v kateri se objekt1 in objekt 2 stikata
	Point *connection_point;
	//v katerih tockah je stik
	int object1_vertex, object2_vertex;
	//kje se stika, povedano v eni spremenljivki
	int ct;


public:
	//konstruktorcek
	Connection(Object *o1, Object *o2, int _ct);
	//destructor
	~Connection();
	//virtual iz primitive
	void draw(FXDCWindow *hdc);
	//virtual iz primitive
	unsigned int get_distance(Point *tocka);
	//virtual iz primitive
	unsigned char on_object(Point *tocka);
	
	
	//vrne tocko povezave
	Point *get_connection_point();
	//kot argument sprejema stevilko objekta v stiku: 1 ali 2
	int get_connected_vertex_number(int obj);
	int get_connected_vertex_number(Line *l);
	int get_connected_vertex_number(Object *o);
	
	//vrne 1, ce je tocka vertex crte o del stikanja
	int is_part_of_connection(int vertex, Object *o);
	int is_part_of_connection(int vertex, Line *l);
	
	//glede na ct nastavi object1_vertex in object2_vertex
	void set_object_vertices();
	//vrne ct, prej pa preveri, ce sta se tocki zamenjali
	int get_ct(Line *l1, Line *l2);

	//preveri, ce je line starejsa od obeh v paru
	int check_line_by_age(Line *line);
	int check_line_by_age(Object *o);
	int check_line_by_age(int i);

	//omejitev se doda k objektom, ki jih omejuje
	void add_to_objects();
	//izbris omejitve
	void remove_from_objects();
};


// *************************************
//
//	Razred PARALLEL
//
// *************************************
class Parallel: public Constraint {
private:
	//kje je center objekta1 in objekta 2
	Point *center_point1, *center_point2;
	//tip paralelnosti
	unsigned char parallel_type;
public:
	//konstruktorcek
	Parallel(Object *o1, Object *o2);
	//destruktor
	~Parallel();
	//virtual iz primitive
	void draw(FXDCWindow *hdc);
	//virtual iz primitive
	unsigned int get_distance(Point *tocka);
	//virtual iz primitive
	unsigned char on_object(Point *tocka);
	//vrne tip stika
	
	void add_to_objects();
	void remove_from_objects();
};


// *************************************
//
//	Razred LENGTH
//
// *************************************
class Length: public Constraint {
public:
	Length(Object *o1, Object *o2);
	~Length();
	//virtual iz Constraint
	void set_temp_change(Point *dp){};
	//virtual iz Constraint
	void move_to_temp_change(){};
	//virtual iz Constraint
	void clear_temp_change(){};
	void add_to_objects();
	void remove_from_objects();
};


//*****************
//horizontal
//*****************

class Horizontal : public Constraint {
public:
	Horizontal(Object *o);
	~Horizontal();
	//virtual iz Constraint
	void set_temp_change(Point *dp){};
	//virtual iz Constraint
	void move_to_temp_change(){};
	//virtual iz Constraint
	void clear_temp_change(){};
	void add_to_objects();
	void remove_from_objects();
};

//****************
//vertical
//****************

class Vertical : public Constraint{
public:
	Vertical(Object *o);
	~Vertical();
	//virtual iz Constraint
	void set_temp_change(Point *dp){};
	//virtual iz Constraint
	void move_to_temp_change(){};
	//virtual iz Constrainta
	void clear_temp_change(){};
	void add_to_objects();
	void remove_from_objects();
};

#endif //_CONSTARINT_H_