/* llist.h
 * Copyright (C) 2004 Djukic Marko, Pernus Klemen, Zagar Miha
 * For conditions of distribution and use, see the copyright notice in main.cpp
 */
#ifndef _LLIST_H_
#define _LLIST_H_

template <class T> class LList;

template <class M> class LListMember {
public:
	//kazalec na memberja
	M* member; 	
	//kazalec na seznam kateremu pripada
	LList<M> *parent;
	//kazalca na prejsnjega in naslednjega
	LListMember<M> *prev,*next; 
	
	//konstruktor ... doda nov element v seznam
	LListMember(M* mem);

	//izbrise element iz seznama glede na del_member
	//ce mode == 1 pusti element pri izbrise tudi memberja
	//drugace pusti memberrja pri miru
	void del(unsigned char delete_member = 1);
};



template <class T> class LList {
public:
	//vrne predhodni elemnt v seznamu
	LListMember<T>* get_prev();
	//vrne naslednji element v seznamu
	LListMember<T>* get_next();
	LListMember<T> *first, *current, *last;
	int elt_count, max_elt_num;
	LList();

	//doda element v llist
	void add(T* elt);
	//izbrise trenutni element glede na del_memeber
	//ce mode == 1 izbrise tudi memberja
	//drugace memberja pusti pri miru
	void del(unsigned char delete_member = 1); 
	//vrne prvi element v seznamu
	LListMember<T>* get_first();
	//vrne trenutni element v seznamu
	//LListMember<T>* get_current();
	//vrne zadnji element v seznamu
	LListMember<T>* get_last();
	//vrne predodni element od mem
	LListMember<T>* get_prev(LListMember<T> *mem);
	//vrne naslennju element od mem
	LListMember<T>* get_next(LListMember<T> *mem);
};








//konstruktor za llistMember
template <class M> LListMember<M>::LListMember(M* mem)
{
	member = mem;
	prev = next = NULL;
}

template <class M> void LListMember<M>::del(unsigned char delete_member)
{
	parent->current = this;
	parent->del(delete_member);
}





//konstruktor ki ne doda nobenega elementa
template <class T> LList<T>::LList()
{
	first = current = last = NULL; //trenutno ni nobenih memberjev
}

template <class T> void LList<T>::add(T* elt)
{
	LListMember<T> *tmp = new LListMember<T>(elt); //ustvarim ListMemberja

	tmp->parent = this;

	if (first == NULL) { //potem je seznam prazen
		first = last = tmp; //vsi kazejo na memberja
	} else {
		tmp->prev = last;	//popravim kazalce
		last->next = tmp;
		last = tmp;
	}
}

template <class T> void LList<T>::del(unsigned char delete_member)
{
	LListMember<T> *tmp = current; //dobim trenuto izbrani element

	if (first == current && last == current) { //samo en element
		if (delete_member) 
			delete current->member;
		delete current;
		first = last = current = NULL;
		return;
	}

	if (first == current) { //sem na prvem elementu
		first = current->next;
		current->next->prev = NULL;
		if (delete_member)
			delete current->member;
		delete current;
		current = NULL;
		return;
	}

	if (last == current) { //sem na zadnjem elementu
		last = current->prev;
		current->prev->next = NULL;
		if (delete_member)
			delete current->member;
		delete current;
		current = NULL;
		return;
	}

	//ce pridem do sem sem nekje na sredini seznama
	current->prev->next = current->next;
	current->next->prev = current->prev;
	if (delete_member)
		delete current->member;
	delete current;
	current = NULL;

	return;
}

template <class T> LListMember<T>* LList<T>::get_first()
{
	current = first;

	return current;
}

/*template <class T> LListMember<T>* LList<T>::get_current()
{
	return current;
}*/

template <class T> LListMember<T>* LList<T>::get_last()
{
	current = last;

	return current;
}

template <class T> LListMember<T>* LList<T>::get_next()
{
	if (current != NULL)
		current = current->next;
	
	return current;
}

template <class T> LListMember<T>* LList<T>::get_next(LListMember<T> *mem)
{	
	current = mem;

	return get_next();
}

template <class T> LListMember<T>* LList<T>::get_prev()
{
	if (current != NULL)
		current = current->prev;

	return current;
}

template <class T> LListMember<T>* LList<T>::get_prev(LListMember<T> *mem)
{	
	current = mem;
	return get_prev();
}

#endif //_LLIST_H_
