/* constraints.cpp
 * Copyright (C) 2004 Djukic Marko, Pernus Klemen, Zagar Miha
 * For conditions of distribution and use, see the copyright notice in main.cpp
 */
#include "constraints.h"

#include <iostream>
using namespace std;
#define ABS(a) (((a) < 0) ? -(a) : (a))

// *****************************************
//
//	Razred CONSTRAINT ... oce vsem omejitvam
//
// *****************************************
Constraint::Constraint() {
	uveljavljen = 0;
	broken = 0;
}

Constraint::~Constraint()
{
}

Object *Constraint::get_object(int obj) {
	switch (obj) {
		case 1:
			return object1;
			break;
		case 2:
			return object2;
			break;
	}

	return NULL;
}


int Constraint::get_constraint_type(){
	return type;
}


Object *Constraint::get_other_object(Object *o) {
	if (o == object1)
		return object2;
	else
		return object1;
}
class Line *Constraint::get_other_line(class Line *l) {
	if (l == line1)
		return line2;
	else
		return line1;
}

// *************************************
//
//	Razred CONNECTION
//
// *************************************
Connection::Connection(Object *o1, Object *o2, int _ct)
{
	object1 = o1;
	object2 = o2;
	line1 = (Line*) object1;
	line2 = (Line*) object2;
	
	type = 1;

	ct = _ct;
	this->set_object_vertices();

	connection_point = new Point(line1->get_vertex_point(object1_vertex));
}

void Connection::draw(FXDCWindow *dc) {
	int epsilon = 8;
	connection_point->move_to(line1->get_vertex_point(object1_vertex));

	dc->setForeground(FXRGB(255,0,0));
	if (selected)
		dc->setLineWidth(2);
	else
		dc->setLineWidth(1);

	dc->drawArc(connection_point->x - epsilon,connection_point->y - epsilon, 2*epsilon, 2*epsilon, 0,23040);
}

unsigned int Connection::get_distance(Point *tocka) {
	return ABS((signed)diff(tocka,connection_point) - 5);
}

unsigned char Connection::on_object(Point *tocka) {
	unsigned int delta = 2;

	if (get_distance(tocka) <= delta)
		return 1;

	return 0;
}

Point *Connection::get_connection_point()
{
	return connection_point;
}

int Connection::get_connected_vertex_number(int obj) {
	if (obj == 1)
		return object1_vertex;
	else if (obj == 2)
		return object2_vertex;
	else
		return -1;
}

int Connection::get_connected_vertex_number(Line *l)
{
	if (l == line1)
		return object1_vertex;
	else if (l == line2)
		return object2_vertex;
	else
		return -2;
}

int Connection::get_connected_vertex_number(Object *o)
{
	if (o == object1)
		return object1_vertex;
	else if (o == object2)
		return object2_vertex;
	else
		return -3;
}

int Connection::is_part_of_connection(int vertex, Object *o)
{
	if (vertex == get_connected_vertex_number(o))
	{
		return 1;
	}
	else
	{
		return 0;
	}
}

int Connection::is_part_of_connection(int vertex, Line *l)
{
	if (vertex == get_connected_vertex_number(l))
	{
		return 1;
	}
	else
	{
		return 0;
	}
}

void Connection::set_object_vertices()
{
	switch(ct)
	{
	case 1:
		object1_vertex = 1;
		object2_vertex = 1;
		break;
	case 2:
		object1_vertex = 1;
		object2_vertex = 2;
		break;
	case 3:
		object1_vertex = 2;
		object2_vertex = 1;
		break;
	case 4:
		object1_vertex = 2;
		object2_vertex = 2;
		break;
	}
}

int Connection::get_ct(Line *l1, Line *l2)
{
	return ct;
}

void Connection::add_to_objects()
{
	object1->add_constraint(this);
	object2->add_constraint(this);
}

void Connection::remove_from_objects()
{
	object1->break_constraint(this);
	object1->break_constraint(this);
}

// *************************************
//
//	Razred PARALLEL
//
// *************************************
Parallel::Parallel(Object *o1, Object *o2) {
	object1 = o1;
	object2 = o2;

	line1 = (Line*) object1;
	line2 = (Line*) object2;
	center_point1 = line1->get_center_of_line();
	center_point2 = line2->get_center_of_line();

	//povem da je tip omejitve parallel
	type = 2;
}

void Parallel::draw(FXDCWindow *dc) {
/*	center_point1 = line1->get_center_of_line();
	center_point2 = line2->get_center_of_line();
	dc->setForeground(FXRGB(0,0,255));
	if (selected)
		dc->setLineWidth(2);
	else
		dc->setLineWidth(1);

	dc->drawLine(center_point1->x,center_point1->y,center_point2->x,center_point2->y);
*/
}

unsigned int Parallel::get_distance(Point *tocka) {
	return (new Line(center_point1,center_point2))->get_distance(tocka);
}

unsigned char Parallel::on_object(Point *tocka) {
	unsigned int delta = 2;

	if (get_distance(tocka) <= delta)
		return 1;

	return 0;
}

void Parallel::add_to_objects()
{
	object1->add_constraint(this);
	object2->add_constraint(this);
}

void Parallel::remove_from_objects()
{
	object1->break_constraint(this);
	object1->break_constraint(this);
}

// *************************************
//
//	Razred LENGTH
//
// *************************************

Length::Length(Object *o1, Object *o2)
{
		object1 = o1; 
		object2 = o2; 
		line1 = (Line*) o1;
		line2 = (Line*) o2;
		type = 3;
}

Length::~Length()
{
}

void Length::add_to_objects()
{
	object1->add_constraint(this);
	object2->add_constraint(this);
}

void Length::remove_from_objects()
{
	object1->break_constraint(this);
	object1->break_constraint(this);
}

// ********************
//
// Razred Horizontal
//
// ********************

Horizontal::Horizontal(Object *o)
{
	object1 = o;
	object2 = NULL;
	line1 = (Line*)object1;
	line2 = NULL;
	type = 4;
}

Horizontal::~Horizontal()
{
}

void Horizontal::add_to_objects()
{
	object1->add_constraint(this);
}

void Horizontal::remove_from_objects()
{
	object1->break_constraint(this);
}

// ********************
//
// Razred Vertical
//
// ********************

Vertical::Vertical(Object *o)
{
	object1 = o;
	object2 = NULL;
	line1 = (Line*)object1;
	line2 = NULL;
	type = 5;
}

Vertical::~Vertical()
{
}

void Vertical::add_to_objects()
{
	object1->add_constraint(this);
}

void Vertical::remove_from_objects()
{
	object1->break_constraint(this);
}