/* objects.cpp
 * Copyright (C) 2004 Djukic Marko, Pernus Klemen, Zagar Miha
 * For conditions of distribution and use, see the copyright notice in main.cpp
 */
#include "objects.h"
#define ABS(a) (((a) < 0) ? -(a) : (a)) 


#include <iostream>


// *************************************
//
//	Razred OBJECT
//
// *************************************
Object::Object() {
	object_width = 1;
	object_color = new Color(0,0,0);

	text = (char *)malloc(sizeof(char) * 300); //OK 300 znakov ne bo noben napisal na linijo
	strcpy(text,"");

	changed = 0;
	recheck = 0;
	already_moved = 0;
}

Object::~Object()
{
}

void Object::set_object_width(unsigned char w) {
	if (w == 0) {
		object_width = 1;
	} else {
		object_width = w;
	}
}

unsigned char Object::get_object_width() {
	return object_width;
}

void Object::set_object_color(Color *c) {
	object_color->set_color(c->r,c->g,c->b);
}

Color *Object::get_object_color() {
	return object_color;
}

void Object::clear_visited_flag()
{
	
	list<Constraint*>::iterator iter_cn;
	Constraint *cn;
	
	for (iter_cn = constraints.begin(); iter_cn != constraints.end(); iter_cn++)
	{
		cn->set_visited_on_move(0);
	}
}

void Object::break_constraint(Constraint *c)
{
	constraints.remove(c);
}

void Object::add_constraint(Constraint *c)
{
	constraints.push_back(c);
}




// *************************************
//
//	Razred LINE
//
// *************************************
Line::Line(Point *t1, Point *t2) {
	//ustvarim koordinati crte
	T1 = new Point(t1->x,t1->y);
	T2 = new Point(t2->x,t2->y);

	changed = 0;
}

Line::~Line()
{
	delete T1;
	delete T2;
}

//virtual iz object
int Line::move_object_by(Point *dp) {
	T1->move_by(dp);
	T2->move_by(dp);
	return 1;
}

//virtual iz object
int Line::move_vertex_by(unsigned char vertex, Point *dp) {
	switch (vertex) {
		case 1:
			T1->move_by(dp);
			break;
		case 2:
			T2->move_by(dp);
			break;
	}
	return 1;
}

//virtual iz object
int Line::move_vertex_to(unsigned char vertex, Point *dp) {
	switch (vertex) {
		case 1:
			T1->move_to(dp);
			break;
		case 2:
			T2->move_to(dp);
			break;
	}
	return 1;
}

//virtual iz object
Point *Line::get_vertex_point(unsigned char vertex) {
	switch (vertex) {
		case 1:
			return T1;
			break;
		case 2:
			return T2;
			break;
	}
	return NULL;
}


unsigned char Object::get_closest_vertex() {
	unsigned char tmp = closest_vertex;
	
	closest_vertex = 0;
	
	return tmp;
}

//virtual iz object
unsigned int Line::get_vertex_distance(Point *tocka) {
	unsigned int d1,d2;

	//prevereim razdalje do obeh tock
	d1 = diff(T1,tocka);
	d2 = diff(T2,tocka);

	//nastavim blizji tocka in vrnem najkrajso razdaljo
	if (d1 <= d2) {
		closest_vertex = 1;
		return d1;
	} else {
		closest_vertex = 2;
		return d2;
	}
}

char *Line::get_text() {
	return text;
}

void Line::set_text(char *t) {
	strcpy(text,t);
}


//virtual iz primitive
unsigned int Line::get_distance(Point *tocka) {
	//Algorithm credits to Po-Han Lin http://www.edepot.com
	unsigned int distance, min_distance;
	bool yLonger=false;
	int incrementVal, endVal;
	int shortLen= T2->y - T1->y;
	int longLen= T2->x - T1->x;
	int decInc;
	int j = 0;
	
	//zacenta razdalja je kar razdalja do prve tocke
	min_distance = distance = diff(T1,tocka);
	closest_point->move_to(T1); //torej je tudi najblizja tocka T1

	if (ABS(shortLen) > ABS(longLen)) {
		int swap = shortLen;
		shortLen = longLen;
		longLen = swap;
		yLonger = true;
	}

	endVal = longLen;
	
	if (longLen < 0) {
		incrementVal = -1;
		longLen = -longLen;
	} else 
		incrementVal = 1;

	if (longLen == 0) 
		decInc = 0;
	else 
		decInc = (shortLen << 16) / longLen;

	if (yLonger) {	
		for (int i = 0; i != endVal; i +=  incrementVal) {
			distance = diff(new Point(T1->x + (j >> 16), T1->y + i),tocka);
			if (distance < min_distance) {
				min_distance = distance;
				closest_point->move_to(T1->x + (j >> 16), T1->y + i); //postavim closest_point
			}
			j += decInc;
		}
	} else {
		for (int i = 0; i != endVal; i += incrementVal) {
			distance = diff(new Point(T1->x + i, T1->y + (j >> 16)),tocka);
			if (distance < min_distance) {
				min_distance = distance;
				closest_point->move_to(T1->x + i, T1->y + (j >> 16)); //postavim closest_point
			}

			j += decInc;
		}
	}

	return min_distance;
}

//virtual iz primitive
unsigned char Line::on_object(Point *tocka) {
	unsigned int delta = 4;

	if (get_distance(tocka) <= delta)
	{
		return 1;
	}
	return 0;
}

//virutal iz object
unsigned char Line::on_vertex(Point *tocka) {
	unsigned int delta = 4;

	if (get_vertex_distance(tocka) <= delta)
		return 1;

	closest_vertex = 0;

	return 0;

}

//virtual iz primitive
void Line::draw(FXDCWindow *dc) {	
	int epsilon = 3;
	int l,x,y;
	
	dc->setForeground(FXRGB(object_color->r,object_color->g,object_color->b));
	dc->setLineWidth(object_width);

	dc->drawLine(T1->x, T1->y, T2->x, T2->y);

	if ((text != NULL) && (l = strlen(text)) > 0) { //pomei da je nekaj napisano
		
		x = ((T2->x + T1->x) / 2) - (l * 7) / 2;
		y = (T2->y + T1->y) / 2;
		dc->drawText(x,y,text,l);
	}

	if (selected) {
		dc->setForeground(FXRGB(127,127,127));
		dc->setLineWidth(1);
		//kvadratka na koncu crta
		dc->drawRectangle(T1->x - epsilon, T1->y - epsilon, 2*epsilon, 2*epsilon);
		dc->drawRectangle(T2->x - epsilon, T2->y - epsilon, 2*epsilon, 2*epsilon);
	}
}

float Line::get_angle()
{	
	double angle;
	if (T1->x == T2->x)
		angle = 90;
	else
	{
		double k = (double)(T2->y - T1->y) / (double)(T2->x - T1->x);
		angle = atan(k) * 180 / 3.14159;
	}
	
	
	//atan ga neki serje, zato uporabljam atan2
	if (abs(angle) > 90)
	{
		if (angle > 0)
			angle =- 180;
		else
			angle =+ 180;
	}

	return angle;
}

double Line::get_length()
{
	return sqrt(double((T1->x - T2->x)*(T1->x - T2->x) + (T1->y - T2->y)*(T1->y - T2->y)));
}

Point *Line::get_center_of_line()
{
	return new Point((T1->x + T2->x)/2, (T1->y + T2->y)/2);
}

Point *Line::get_other_vertex_point(int vertex)
{
	if (vertex == 1)
		return T2;
	else if (vertex == 2)
		return T1;
	else
		return NULL;
}